/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.httpinvoker;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import org.unitils.UnitilsJUnit4;
import org.unitils.database.DatabaseUnitils;
import org.unitils.database.annotations.Transactional;
import org.unitils.database.util.TransactionMode;

import eu.etaxonomy.cdm.api.application.CdmApplicationRemoteController;
import eu.etaxonomy.cdm.api.application.ICdmApplicationConfiguration;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.remote.CdmPersistentRemoteSource;
import eu.etaxonomy.cdm.remote.CdmRemoteSource;
import eu.etaxonomy.cdm.remote.CdmRemoteSourceException;
import eu.etaxonomy.cdm.remote.ICdmRemoteSource;


/**
 * Base class for remoting tests, responsible for
 * - starting / stop the cdm server
 * - running some basic connection tests
 * - setting up the remote configuration.
 *
 */
@Transactional(TransactionMode.DISABLED)
public class BaseRemotingTest extends UnitilsJUnit4 {
    private static final Logger logger = Logger.getLogger(BaseRemotingTest.class);

    private static ICdmApplicationConfiguration remoteApplicationController;
    private static ICdmRemoteSource cdmRemoteSource;
    private static CdmPersistentRemoteSource remotePersistentSource;

    private static boolean stoppingServer = false;

    @BeforeClass
    public static void  initializeBaseRemotingTest() {
        Logger.getRootLogger().setLevel(Level.INFO);

        //DatabaseUnitils.disableConstraints();
        try {
            CDMServer.getInstance().start();
        } catch (Exception e) {
            e.printStackTrace();
            Assert.fail("Server failed to start. Reason : " + e.getMessage());
        }
        cdmRemoteSource = CdmRemoteSource.NewInstance(CDMServer.getInstance().getName(),
                CDMServer.getInstance().getHost(),
                CDMServer.getInstance().getPort(),
                CDMServer.getInstance().getContextPath(),
                NomenclaturalCode.ICNAFP);
        remoteApplicationController =
                CdmApplicationRemoteController.NewInstance(cdmRemoteSource,
                        false,
                        null,
                        null);

        try {
            remotePersistentSource = CdmPersistentRemoteSource.NewInstance("default");
        } catch (CdmRemoteSourceException e) {
            Assert.fail("Default Remote Persistent Source failed to load. Reason : " + e.getMessage());
        }

    }


    protected static ICdmApplicationConfiguration getRemoteApplicationController() {
        return remoteApplicationController;
    }

    protected static ICdmRemoteSource getCdmRemoteSource() {
        return cdmRemoteSource;
    }

    protected static CdmPersistentRemoteSource getCdmPersistentRemoteSource() {
        return remotePersistentSource;
    }

    @AfterClass
    public static void cleanup() {
        try {
            CDMServer.getInstance().stop();
        } catch (Exception e) {
            Assert.fail("Server could not be stopped. Reason : " + e.getMessage());
        }
    }


}
